TATWEEL = "ـ"

DIACRITICS_MAPPINGS = {
    "fatha": "َ",
    "damma": "ُ",
    "kasra": "ِ",
    "tanween_fath": "ً",
    "tanween_damm": "ٌ",
    "tanween_kasr": "ٍ",
    "sokoon": "ْ",
    "shadda": "ّ",
}
DIACRITICS = DIACRITICS_MAPPINGS.values()

WESTERN_ARABIC_DIGITS = (
    0,
    1,
    2,
    3,
    4,
    5,
    6,
    7,
    8,
    9,
)
INDO_ARABIC_DIGITS = (
    "٠",
    "١",
    "٢",
    "٣",
    "٤",
    "٥",
    "٦",
    "٧",
    "٨",
    "٩",
)
DIGITS = WESTERN_ARABIC_DIGITS + INDO_ARABIC_DIGITS

ALEF_HAMZA_FORMS = (
    "أ",
    "إ",
    "آ",
)
NON_ALIF_HAMZA_FORMS = (
    "ؤ",
    "ئ",
)
HAMZA = "ء"
HAMZA_LETTERS = ALEF_HAMZA_FORMS + NON_ALIF_HAMZA_FORMS + (HAMZA,)

TA_MARBUTA = "ة"
ALEF_MADDA = ""
ALEF_WASLA = "ٱ"
ALIF_MAQSURA = "ى"

ALIF_LETTERS = (
    ALEF_WASLA,  # Alef wasla
    ALIF_MAQSURA,  # Alef mqsoura
)

BASIC_LETTERS = (
    "ا",
    "ب",
    "ت",
    "ث",
    "ج",
    "ح",
    "خ",
    "د",
    "ذ",
    "ر",
    "ز",
    "س",
    "ش",
    "ص",
    "ض",
    "ط",
    "ظ",
    "ع",
    "غ",
    "ف",
    "ق",
    "ك",
    "ل",
    "م",
    "ن",
    "ه",
    "و",
    "ي",
)

LETTERS = BASIC_LETTERS + HAMZA_LETTERS + ALIF_LETTERS + (TA_MARBUTA,)

NUMBERS_PUNCTUATION_MARKS = ("ر",)
ARABIC_PUNCTUATION_MARKS = ("،", "؟", "؛")
OTHER_PUNCTUATION_MARKS = (
    ",",
    "~",
    "^",
    "(",
    "%",
    ")",
    "'",
    "!",
    "÷",
    "\\",
    ".",
    "@",
    "=",
    "¡",
    "×",
    "<",
    "¿",
    "$",
    "º",
    "&",
    "#",
    ">",
    "*",
    "_",
    "|",
    "+",
    ":",
    "ø",
    ";",
    "-",
)

PUNCTUATION_MARKS = (
    ARABIC_PUNCTUATION_MARKS + OTHER_PUNCTUATION_MARKS + NUMBERS_PUNCTUATION_MARKS
)

PREFIXES = (
    "ب",
    "و",
    "ا",
    "ي",
    "س",
    "ت",
    "ن",
    "م",
    "ف",
    "ل",
    "لل",
)
SUFFIXES = (
    "ا",
    "تي",
    "ن",
    "تن",
    "كن",
    "وا",
    "ين",
    "هما",
    "ون",
    "ان",
    "ما",
    "ك",
    "نا",
    "ته",
    "ات",
    "ة",
    "تما",
    "هن",
    "ه",
    "كم",
    "ت",
    "ي",
    "ني",
    "كما",
    "ها",
    "تا",
    "هم",
    "تم",
)



import re


def pattern_sub(text: str, replacements: dict) -> str:
    """An alternative to "str".replace

    This function replaces multiple strings in a text using regex instead
    of looping of replacements.

    Args:
        text (str): The text we need to replace parts (or all) of it.
        replacements (dict): A map of all substrings and their replacements.

    Returns:
        str: The final text with replacements.
    """
    # use these three lines to do the replacement
    replacements = dict((re.escape(k), v) for k, v in replacements.items())

    pattern = re.compile("|".join(replacements.keys()))
    return pattern.sub(lambda m: replacements[re.escape(m.group(0))], text)



from matn import elements as ase
from matn.normalizers import clean_text

jummal_mappings = {
    "ا": 1,
    "ء": 1,  # Hamza
    "أ": 1,  # Hamza on alef maftoha
    "إ": 1,  # Hamza on alef maksora
    "ب": 2,
    "ج": 3,
    "د": 4,
    "ه": 5,
    "ة": 5,  # Teh marbota
    "و": 6,
    "ؤ": 6,  # Hamza on waw
    "ز": 7,
    "ح": 8,
    "ط": 9,
    "ي": 10,
    "ى": 10,  # Alef maksora
    "ئ": 10,  # Hamza on yeh
    "ك": 20,
    "ل": 30,
    "م": 40,
    "ن": 50,
    "س": 60,
    "ع": 70,
    "ف": 80,
    "ص": 90,
    "ق": 100,
    "ر": 200,
    "ش": 300,
    "ت": 400,
    "ث": 500,
    "خ": 600,
    "ذ": 700,
    "ض": 800,
    "ظ": 900,
    "غ": 1_000,
}


def jummal(
    text: str,
    use_hamza: bool = False,
    use_tarkeeb: bool = False,
    normalize_hamza: bool = False,
) -> int:
    """Get the decimal alphanumeric code of a given string.

    Or Abjad numerals, a decimal alphabetic numeral system/alphanumeric code,
    in which the 28 letters of the Arabic alphabet are assigned numerical
    values. They have been used in the Arabic-speaking world since before
    the eighth century when positional Arabic numerals were adopted.

    Args:
        text (str): Text to get its jummal.
        use_hamza (bool, optional): Count hamza as an alef. Defaults to False.
        use_tarkeeb (bool, optional): Used to express the numbers from 2000 to
            1,000,000, using the rule based on the letter "غ". The rule says: any
            character that comes before "غ" its value will be multiplied with
            1000 instead of accumalated to it. Defaults to False.
        normalize_hamza (bool, optional): Treat all hamza forms as a regular alef
            instead of the letter it appears on. Defaults to False.

    Returns:
        int: The jummal count of the given string
    """
    text = clean_text(text)

    if not text:
        return 0

    count = 0
    total = len(text)

    skip = False
    for i, char in enumerate(text):
        if skip:
            skip = False
            continue

        if not use_hamza and char == ase.HAMZA:
            continue

        if use_tarkeeb and not char.isspace() and i + 1 < total and text[i + 1] == "غ":
            count += 1000 * jummal_mappings[char]
            skip = True
            continue

        if normalize_hamza and char in ase.HAMZA_LETTERS:
            count += 1
            continue

        count += jummal_mappings.get(char, 0)

    return count


def word_count(text: str, split_badama: str = False) -> int:
    """Counts the number of words in a given string.

    Args:
        text (str): The text to count its words.
        split_badama (str, optional): Wether to count the
            word بعدما as two words بعد and ما. Defaults to False.

    Returns:
        int: The count of words in a given string.
    """
    text = clean_text(text)
    if not text:
        return 0

    words = text.split()
    count = len(words)

    if split_badama:
        count += text.count("بعدما")

    return count


def char_count(
    text: str, include_spaces: bool = False, hamza_madda: bool = False
) -> int:
    """Counts the number of characters in a given string.

    Args:
        text (str): The text to count its characters.
        include_spaces (bool, optional): Indicates whether spaces should be
            included in the count or not. Defaults to False.
        hamza_madda (bool, optional): Whether to consider the hamza madda (أٓ)
            two characters or not. Defaults to False.

    Returns:
        int: The number of characters found in the given string.
    """
    if not text:
        return 0

    clean = clean_text(text)
    count = len(clean)

    if hamza_madda:
        count += text.count("أٓ")

    if not include_spaces:
        count -= clean.count(" ")

    return count



import argparse

from matn.counters import char_count, jummal, word_count


def get_args():
    parser = argparse.ArgumentParser(description=("Arabic text processor tool."))

    subparsers = parser.add_subparsers(
        dest="processor", help="the processor to be applied on the given string"
    )

    parser_jummal = subparsers.add_parser("jummal", help="Jummal counter")
    parser_jummal.add_argument(
        "--use-hamza", "-z", action="store_true", help="count hamza as an alef."
    )
    parser_jummal.add_argument(
        "--normalize-hamza",
        "-n",
        action="store_true",
        help=(
            "Treat all hamza forms as a regular alef instead of the "
            "letter it appears on. Defaults to False."
        ),
    )
    parser_jummal.add_argument(
        "--use-tarkeeb",
        "-t",
        action="store_true",
        help=(
            "the tarkeeb rule based on the letter 'غ' to express "
            "numbers from 2,000 to 1,000,000."
        ),
    )

    parser_wc = subparsers.add_parser("wc", help="Words counter")
    parser_wc.add_argument(
        "--split-badama",
        "-s",
        action="store_true",
        help="wether to count the word بعدما as two words بعد and ما.",
    )

    parser_cc = subparsers.add_parser("cc", help="Characters counter")
    parser_cc.add_argument(
        "--hamza-madda",
        "-m",
        action="store_true",
        help="indicates if spaces should be included in the count.",
    )
    parser_cc.add_argument(
        "--include-spaces",
        "-s",
        action="store_true",
        help="consider the hamza madda (أٓ) two characters.",
    )

    parser.add_argument(
        "text",
        type=str,
        help="the string to process",
    )

    return parser, parser.parse_args()


def main():
    parser, args = get_args()

    if args.processor == "jummal":
        print(
            jummal(
                args.text,
                use_hamza=args.use_hamza,
                use_tarkeeb=args.use_tarkeeb,
                normalize_hamza=args.normalize_hamza,
            )
        )
    elif args.processor == "wc":
        print(word_count(args.text, split_badama=args.split_badama))
    elif args.processor == "cc":
        print(
            char_count(
                args.text,
                include_spaces=args.include_spaces,
                hamza_madda=args.hamza_madda,
            )
        )
    else:
        parser.error("Prcessor action not recognized")


if __name__ == "__main__":
    exit(main())



import matn.elements as ase


def tatweel_removal(text: str) -> str:
    """Remove kashidas from string

    The Tatweel (elongation, or kashidas) is used to stretch words
    to indicate prominence or simply to force vertical justification.
    This symbol has no effect on the meaning of the word so it's
    usually normalized.

    Examples:
        - A word without Tatweel:       جميل
        - The same word with Tatweel:   جـــمـــيـــل

    Args:
        text (str): The text we need to extract the Tatweel from.

    Returns:
        str: A text without Tatweels.
    """
    if not text:
        return text

    return text.replace(ase.TATWEEL, "")


def diacritic_removal(text: str) -> str:
    """Remove diacritics from string

    Since diacritics occur so infrequently, they are considered noise
    by most researchers and are simply removed from the text.

    Examples:
        - A word without diacritics:     جميل
        - The same word with diacritics: جَمِيلٌ

    Args:
        text (str): The text that we need to extract diacritics from.

    Returns:
        str: A text without diacritics.
    """
    if not text:
        return text

    for diacritic in ase.DIACRITICS:
        text = text.replace(diacritic, "")

    return text


def punctuation_removal(text: str) -> str:
    """Remove all punctuation marks from text

    Args:
        text (str): The text to process.

    Returns:
        str: A punctuation-free text
    """
    if not text:
        return text

    for mark in ase.PUNCTUATION_MARKS:
        if mark in ase.NUMBERS_PUNCTUATION_MARKS:
            continue

        text = text.replace(mark, "")

    return text


def letter_normalization(text: str, egyptian: bool = False) -> str:
    """Normalize charactes in a given string to a standard form
    There are four letters in Arabic that are so often misspelled using
    variants that researchers find it more helpful to completely make
    these variants ambiguous (normalized).

    1. The Hamzated forms of Alif -> Alif.
    2. The Alif-Maqsura -> Ya (Only in Egypt).
    3. The Ta-Marbuta -> Ha.
    4. The non-Alif forms of Hamza -> Hamza letter.

    However, this is sometimes may be problematic. Let's take the
    name 'Ana' and the word 'Me' meaning for example, both words after
    normalization are gonna produce the same word which's not going
    to be interesting especially in named entity recognition.

     Examples:
       * Ana:
        - Correct form: آنا
        - After Normalization: انا
       * Me:
        - Correct form: أنا
        - After Normalization: انا

    Args:
        text (str): The text we want to normalize its letters.
        egyptian (bool, optional): To flag if we want to normalize the
            Alif-Maqsura. Defaults to False.

    Returns:
        str:  A letter-normalized string
    """
    if not text:
        return text

    if egyptian:
        text = text.replace(ase.ALIF_MAQSURA, "ي")

    for form in ase.ALEF_HAMZA_FORMS:
        text = text.replace(form, "ا")

    text = text.replace(ase.TA_MARBUTA, "ه")

    for form in ase.NON_ALIF_HAMZA_FORMS:
        text = text.replace(form, ase.HAMZA)

    return text


def clean_text(text: str) -> str:
    """Cleans the word by removing punctuations, diacritics, non-letter
    characters.

    Args:
        text (str): The word to clean

    Returns:
        str: A cleaned word that has nothing but letters.
    """

    if not text:
        return text

    # Remove whitespace characters from the beginning and the end
    text = text.strip()

    for letter in text:
        if letter not in ase.LETTERS and letter != " ":
            text = text.replace(letter, "")

    return text



#!/usr/bin/env python
from pathlib import Path

from setuptools import setup

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="matn",
    version="0.2.0",
    packages=["matn"],
    install_requires=[],
    entry_points={
        "console_scripts": [
            "matn = matn.cli:main",
        ],
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: MIT License",
        "Topic :: Software Development",
    ],
    url="https://github.com/iamjazzar/matn",
    license="MIT License",
    author="Ahmed Jazzar",
    author_email="me@ahmedjazzar.com",
    description="Arabic text processor tool.",
    long_description=long_description,
    long_description_content_type="text/markdown",
)
